using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using BlogML;
using BlogML.Xml;

namespace Rss2BlogML
{
	public class BlogMLWriter : BlogMLWriterBase
	{

		BlogMLBlog _bmlBlog;

		public BlogMLWriter(BlogMLBlog blog)
		{
			_bmlBlog = blog;
		}

		protected override void InternalWriteBlog()
		{
			WriteStartBlog(_bmlBlog.Title, _bmlBlog.SubTitle, _bmlBlog.RootUrl, _bmlBlog.DateCreated);

			WriteAuthors();
			WriteCategories();

			WritePosts();

			WriteEndElement(); // End Blog Element
			Writer.Flush();
		}

		private void WritePosts()
		{
			WriteStartPosts();

			foreach (BlogMLPost bmlPost in _bmlBlog.Posts)
				WritePost(bmlPost);

			WriteEndElement(); // </posts>
		}

		private void WritePost(BlogMLPost bmlPost)
		{
			string postId = bmlPost.ID;
			WriteStartPost(postId, bmlPost.Title, bmlPost.DateCreated, bmlPost.DateModified, bmlPost.Approved, bmlPost.Content.Text,
						   bmlPost.PostUrl, bmlPost.Views, bmlPost.PostType, bmlPost.PostName);

			WritePostCategories(bmlPost.Categories);
			WritePostComments(bmlPost.Comments);
			WritePostTrackbacks(bmlPost.Trackbacks);
			WritePostAttachments(bmlPost);
			WritePostAuthors(bmlPost.Authors);

			WriteEndElement();	// </post>
		}

		private void WritePostAuthors(BlogMLPost.AuthorReferenceCollection authorsRefs)
		{
			if (authorsRefs.Count > 0)
			{
				WriteStartAuthors();
				foreach (BlogMLAuthorReference authorRef in authorsRefs)
				{
					WritePostAuthor(authorRef.Ref);
				}
				WriteEndElement();
			}
		}

		private void WritePostAuthor(string authorId)
		{
			string authorRef = authorId;
			WriteAuthorReference(authorRef);
		}

		private void WritePostAttachments(BlogMLPost bmlPost)
		{
			if (bmlPost.Attachments.Count > 0)
			{
				WriteStartAttachments();
				foreach (BlogMLAttachment attachment in bmlPost.Attachments)
				{
					if (attachment.Embedded)
					{
						WriteAttachment(attachment.Url, attachment.Data.Length, attachment.MimeType, attachment.Path, attachment.Embedded, attachment.Data);
					}
					else
					{
						WriteAttachment(attachment.Path, attachment.MimeType, attachment.Url);
					}
					//Writer.Flush();
				}
				WriteEndElement(); // End Attachments Element
				Writer.Flush();
			}
		}

		private void WritePostTrackbacks(BlogMLPost.TrackbackCollection trackbacks)
		{
			if (trackbacks.Count > 0)
			{
				WriteStartTrackbacks();
				foreach (BlogMLTrackback bmlTrackback in trackbacks)
				{
					string trackBackId = bmlTrackback.ID;
					WriteTrackback(trackBackId, bmlTrackback.Title, ContentTypes.Text, bmlTrackback.DateCreated, bmlTrackback.DateModified, bmlTrackback.Approved, (bmlTrackback.Url ?? string.Empty));
				}
				WriteEndElement();
			}
		}

		private void WritePostComments(BlogMLPost.CommentCollection comments)
		{
			if (comments.Count > 0)
			{
				WriteStartComments();
				foreach (BlogMLComment bmlComment in comments)
				{
					WritePostComment(bmlComment);
				}
				WriteEndElement();
			}
		}

		private void WritePostComment(BlogMLComment bmlComment)
		{
			string commentId = bmlComment.ID;
			string userName = string.IsNullOrEmpty(bmlComment.UserName) ? "Anonymous" : bmlComment.UserName;
			WriteComment(commentId, bmlComment.Title, ContentTypes.Text, bmlComment.DateCreated, bmlComment.DateModified, bmlComment.Approved, userName, bmlComment.UserEMail, bmlComment.UserUrl, bmlComment.Content.Text, ContentTypes.Text);
		}

		protected void WritePostCategories(BlogMLPost.CategoryReferenceCollection categoryRefs)
		{
			if (categoryRefs.Count > 0)
			{
				WriteStartCategories();
				foreach (BlogMLCategoryReference categoryRef in categoryRefs)
				{
					WritePostCategory(categoryRef.Ref);
				}
				WriteEndElement();
			}
		}

		private void WritePostCategory(string categoryId)
		{
			string categoryRef = categoryId;
			WriteCategoryReference(categoryRef);
		}

		private void WriteCategories()
		{
			WriteStartCategories();
			foreach (BlogMLCategory bmlCategory in _bmlBlog.Categories)
			{
				string categoryId = bmlCategory.ID;
				string parentId = bmlCategory.ParentRef;
				WriteCategory(categoryId, bmlCategory.Title, ContentTypes.Text, bmlCategory.DateCreated, bmlCategory.DateModified, bmlCategory.Approved, bmlCategory.Description, parentId);
			}
			WriteEndElement();
		}

		private void WriteAuthors()
		{
			WriteStartAuthors();
			foreach (BlogMLAuthor bmlAuthor in _bmlBlog.Authors)
			{
				WriteAuthor(
					bmlAuthor.ID,
					bmlAuthor.Title,
					bmlAuthor.Email,
					bmlAuthor.DateCreated,
					bmlAuthor.DateModified,
					bmlAuthor.Approved);
			}
			WriteEndElement(); // </authors>
		}
	}
}
