using System;
using System.Collections.Generic;
using System.Text;
using RssToolkit;
using BlogML.Xml;

namespace Rss2BlogML
{
	public class RssGrabber
	{

		private IRssMapper _rssMapper;
		private BlogMLBlog _blog;

		internal RssGrabber(IRssMapper rssMapper, BlogMLBlog blog)
		{
			_rssMapper = rssMapper;
			_blog = blog;
		}

		public void Grab(string feedUrl, bool isMainFeed)
		{
			GenericRssChannel c = GenericRssChannel.LoadChannel(feedUrl);
			if (isMainFeed)
			{
				_blog.Title = Parsers.ParseString(c.Attributes, "title");
				_blog.SubTitle = Parsers.ParseString(c.Attributes, "description");
				_blog.RootUrl = Parsers.StripDomainFromUrl(Parsers.ParseString(c.Attributes, "link"));
				_blog.DateCreated = DateTime.Now;
			}
			foreach (GenericRssElement rssPost in c.Items)
			{
				string postId = _rssMapper.GeneratePostId(rssPost);

				BlogMLPost post = FindPostById(postId);

				if (post != null)
				{
					IList<BlogMLCategoryReference> categoryList = GetCategoryReferences(_blog, rssPost);
					if (categoryList.Count > 0)
						foreach (BlogMLCategoryReference catRef in categoryList)
						{
							if(!post.Categories.Contains(catRef))
								post.Categories.Add(catRef);
						}
				}
				else
				{
					post = new BlogMLPost();
					post.Approved = true;
					post.HasExcerpt = false;
					post.PostType = BlogML.BlogPostTypes.Normal;
					post.Views = 0;

					BlogMLAuthorReference authorRef = GetAuthorReference(_blog, rssPost);
					if (authorRef != null)
						post.Authors.Add(authorRef);

					IList<BlogMLCategoryReference> categoryList = GetCategoryReferences(_blog, rssPost);
					if (categoryList.Count > 0)
						foreach (BlogMLCategoryReference catRef in categoryList)
						{
							post.Categories.Add(catRef);
						}
					post.ID = postId;
					post.Title = Parsers.ParseString(rssPost.Attributes, "title");
					post.PostName = _rssMapper.GeneratePostName(rssPost);
					post.Content = new BlogMLContent();
					post.Content.Text = Parsers.ParseString(rssPost.Attributes, "description");
					post.PostUrl = Parsers.StripDomainFromUrl(Parsers.ParseString(rssPost.Attributes, "guid"));
					post.Excerpt = new BlogMLContent();
					int commentNumber = Parsers.ParseInt(rssPost.Attributes, "slash:comments");
					if (commentNumber > 0)
						AddComments(post, Parsers.ParseString(rssPost.Attributes, "wfw:commentRss"));
					post.DateCreated = post.DateModified = Parsers.ParseDateTime(rssPost.Attributes, "pubDate");
					_blog.Posts.Add(post);
				}
			}
		}

		private BlogMLPost FindPostById(string postId)
		{
			foreach (BlogMLPost post in _blog.Posts)
			{
				if (post.ID.Equals(postId))
					return post;
			}
			return null;
		}

		private void AddComments(BlogMLPost post, string commentRssUrl)
		{
			if (!String.IsNullOrEmpty(commentRssUrl))
			{
				GenericRssChannel c = GenericRssChannel.LoadChannel(commentRssUrl);
				foreach (GenericRssElement rssPost in c.Items)
				{
					BlogMLComment comment = new BlogMLComment();
					comment.ID = _rssMapper.GenerateCommentId(rssPost);
					comment.UserName = Parsers.ParseString(rssPost.Attributes, "dc:creator");
					comment.Title = Parsers.ParseString(rssPost.Attributes, "title");
					comment.DateCreated = comment.DateModified = Parsers.ParseDateTime(rssPost.Attributes, "pubDate");
					comment.Content = new BlogMLContent();
					comment.Content.Text = Parsers.ParseString(rssPost.Attributes, "description");
					post.Comments.Add(comment);
				}
			}
		}


		private IList<BlogMLCategoryReference> GetCategoryReferences(BlogMLBlog _blog, GenericRssElement rssPost)
		{
			IList<BlogMLCategoryReference> list = new List<BlogMLCategoryReference>();
			CatStruct[] postCategories = _rssMapper.GenerateCategoryIds(rssPost);

			foreach (CatStruct cat in postCategories)
			{
				if(cat.IsNewId)
					list.Add(CreateNewCategory(_blog,cat.Name,cat.Id));
				else
					list.Add(CreateCategoryReference(cat.Id));
			}

			return list;
		}


		private BlogMLCategoryReference CreateCategoryReference(string id)
		{
			BlogMLCategoryReference refId = new BlogMLCategoryReference();
			refId.Ref = id;
			return refId;
		}

		private BlogMLCategoryReference CreateNewCategory(BlogMLBlog blog, string category, string categoryId)
		{
			BlogMLCategory blogCategory = new BlogMLCategory();
			blogCategory.Title = category;
			blogCategory.ID = categoryId;
			blog.Categories.Add(blogCategory);
			return CreateCategoryReference(categoryId);
		}


		private BlogMLAuthorReference GetAuthorReference(BlogMLBlog blog, GenericRssElement rssPost)
		{
			IdStruct authorId = _rssMapper.GenerateAuthorId(rssPost);
			if(authorId.IsNewId)
				return CreateNewBlogAuthor(blog, Parsers.ParseString(rssPost.Attributes, "dc:creator"), authorId.Id);
			else
				return CreateBlogAuthorReference(authorId.Id);
		}

		private BlogMLAuthorReference CreateBlogAuthorReference(string id)
		{
			BlogMLAuthorReference refId = new BlogMLAuthorReference();
			refId.Ref = id;
			return refId;
		}

		private BlogMLAuthorReference CreateNewBlogAuthor(BlogMLBlog blog, string author, string authorId)
		{
			BlogMLAuthor blogAuthor = new BlogMLAuthor();
			blogAuthor.Title = author;
			blogAuthor.ID = authorId;
			blog.Authors.Add(blogAuthor);
			return CreateBlogAuthorReference(authorId);
		}

	}
}
