############################################################
## 
## Script: BloggerToBlogML.ps1
## By: Aaron Lerch
## Website: www.aaronlerch.com/blog
## 
## Saves as much blogger content as one can get into an
## array of BlogML xml strings (one element in the array per blog)
## 
## Usage:  BloggerToBlogML [string] bloggeruserid
## 
## Your user id can be found in the URL for your blogger profile.
## 
############################################################

param ([string] $userid)

$script:MAXBLOGRESULTS = "100"
$script:MAXPOSTRESULTS = "300"
$script:MAXCOMMENTRESULTS = "100"

function Main()
{
    $blogs = @()

    $scriptPath = split-path $MyInvocation.ScriptName
    $blogMLPath = join-path $scriptPath "BlogML.dll"
    [System.Reflection.Assembly]::LoadFile($blogMLPath) | out-null
    
    $blogList = GetBlogList($userid)
    foreach ($entry in $blogList.feed.entry)
    {
        Write-Host Processing blog: $entry.title."#text"
    
        $blogUrl = ($entry.link | where { $_.rel -eq "self" } | select href -first 1 -last 1).href
        $blogPostUrl = ($entry.link | where { $_.rel -eq "http://schemas.google.com/g/2005#post" } | select href -first 1 -last 1).href

		# Create the BlogML objects, populate them, and write it out to a file
		$blogMLBlog = CreateBlogMLBlog $entry
		GetBlogPosts $blogMLBlog $blogPostUrl
		
		# Output the string version of this BlogML instance
		$sw = new-object System.IO.StringWriter
		$w = new-object System.Xml.XmlTextWriter($sw)
		$w.Formatting = [System.Xml.Formatting]::Indented
		[BlogML.Xml.BlogMLSerializer]::Serialize($w, $blogMLBlog)
		
		$blogs += $sw.ToString()
		
		Write-Host Retrieval completed with $blogMLBlog.Posts.Count posts
		
		#$blogs += $blogMLBlog
    }
    
    return $blogs
}

function GetBlogList($userid)
{
    $url = "http://www.blogger.com/feeds/$userid/blogs?start-index=1&max-results=$MAXBLOGRESULTS"
    $blogList = [xml](new-object System.Net.WebClient).DownloadString($url)
    return $blogList
}

function CreateBlogMLBlog($blogEntry)
{
    $blogmlblog = new-object BlogML.Xml.BlogMLBlog
    
    # Add Title
    $blogmlblog.Title = $blogEntry.title."#text"
    
    # Add Author
    $blogmlauthor = new-object BlogML.Xml.BlogMLAuthor
    $blogmlauthor.ID = $blogentry.author.name
    $dummy = $blogmlblog.Authors.Add($blogmlauthor)
    
    # Add Categories
    foreach ($blogcategory in $blogEntry.category)
    {
        $blogmlcategory = new-object BlogML.Xml.BlogMLCategory
        $blogmlcategory.ID = $blogcategory.term
        $dummy = $blogmlblog.Categories.Add($blogmlcategory)
    }
    
    Write-Host Found $blogEntry.category.Length categories
    
    return $blogmlblog
}

function GetBlogPosts($blogmlblog, $blogPostUrl)
{
    if ($blogPostUrl -eq $null)
    {
        throw "A blog post url is required"
    }
    
    Write-Progress -activity "Retrieving Posts" -status "Loading Posts" -percentComplete 0

    $url = $blogPostUrl + "?start-index=1&max-results=$MAXPOSTRESULTS"
    $blogPostList = [xml](new-object System.Net.WebClient).DownloadString($url)
    if ($blogPostList.feed.entry -ne $null -and $blogPostList.feed.entry.Length -gt 0)
    {
        Write-Progress -activity "Retrieving Posts" -status "Processing Posts" -percentComplete 0
    
        $numOfPosts = $blogPostList.feed.entry.Length
        $counter = 1
        foreach ($blogPost in $blogPostList.feed.entry)
        {
            Write-Progress -activity "Retrieving Posts" -status "Processing Posts" -percentComplete ($counter/$numOfPosts*100)
        
		    $blogmlpost = CreateBlogMLPost $blogPost
            $dummy = $blogmlblog.Posts.Add($blogmlpost)
            
            $counter++
        }
        
        Write-Progress -activity "Retrieving Posts" -status "Loading Posts" -completed
    }
    else
    {
        Write-Progress -activity "Retrieving Posts" -status "Loading Posts" -completed
        Write-Host No posts were retrieved
    }
}

function CreateBlogMLPost($blogPost)
{
    $blogid = GetBlogId($blogPost.id)
    $postid = GetPostId($blogPost.id)

    $blogmlpost = new-object BlogML.Xml.BlogMLPost
    $blogmlpost.ID = $postid
    $blogmlpost.DateCreated = $blogPost.published
    $blogmlpost.DateModified = $blogPost.updated
    $blogmlpost.Title = $blogPost.title."#text"
    $blogmlpost.HasExcerpt = $false
    $blogmlpost.PostType = [BlogML.BlogPostTypes]::Normal
    $blogmlpost.PostName = $blogPost.title."#text"
    $blogmlpost.PostUrl = ($blogPost.link | where { $_.rel -eq "alternate" } | select href -first 1 -last 1).href
    $dummy = $blogmlpost.Authors.Add($blogPost.author.name)
    
    # Add categories
    if ($blogPost.category -ne $null -and $blogPost.category.Length -gt 0)
    {
        foreach ($category in $blogPost.category)
        {
            $dummy = $blogmlpost.Categories.Add($category.term)
        }
    }
    
    # Add content
    $blogmlpost.Content.Text = $blogPost.content."#text"
    
    # Add Comments
    $blogpostcommenturl = "http://www.blogger.com/feeds/$blogid/$postid/comments/default"
    GetBlogPostComments $blogmlpost $blogpostcommenturl
        
    return $blogmlpost
}

function GetBlogPostComments($blogmlpost, $blogpostcommenturl)
{
    $url = $blogpostcommenturl + "?start-index=1&max-results=$MAXCOMMENTRESULTS"
    $blogCommentList = [xml](new-object System.Net.WebClient).DownloadString($url)
    
    if ($blogCommentList.feed.entry -ne $null -and $blogCommentList.feed.entry.Length -gt 0)
    {
        foreach ($blogComment in $blogCommentList.feed.entry)
        {
            $blogmlcomment = CreateBlogMLPostComment $blogComment
            $dummy = $blogmlpost.Comments.Add($blogmlcomment)
        }
    }
}

function CreateBlogMLPostComment($blogComment)
{
    $blogmlcomment = new-object BlogML.Xml.BlogMLComment
    $blogmlcomment.Content.Text = $blogComment.content."#text"
    $blogmlcomment.UserName = $blogComment.author.name
    
    return $blogmlcomment
}

function GetPostId($bloggerid)
{
    return GetMatch "tag:blogger\.com.*\.post-([0-9]+)" $bloggerid 1
}

function GetBlogId($bloggerid)
{
    return GetMatch "tag:blogger\.com.*\:blog-([0-9]+)" $bloggerid 1
}

function GetMatch($pattern, $source, $getGroupNum)
{
    $pat = [regex]$pattern
    $match = $pat.Match($source)
    if ($match.Success -eq $true)
    {
        return $match.Groups[$getGroupNum].Value
    }

    return ""
}

. Main
