﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using System.IO;
using BlogML;
using BlogML.Xml;
using CommunityServer;
using CommunityServer.Components;
using CommunityServer.Blogs;
using CommunityServer.Blogs.Components;

namespace CommunityServer.BlogML
{
	public class Reader
	{

		#region Private members
		// Fields
		private Weblog _Blog;
		private int _BlogPostCount = 0;
		private int _BlogAttachmentCount = 0;
		private int _BlogCommentCount = 0;
		private int _BlogTrackBackCount = 0;
		#endregion

		#region Constructor
		// Methods
		public Reader(string ApplicationKey)
		{
			this._Blog = Weblogs.GetWeblog(ApplicationKey);
		}
		#endregion

		#region Main process
		public string LoadBlog(string XML, bool ClearOldItems)
		{
			if (ClearOldItems)
				Clear();

			BlogMLBlog blog = new BlogMLBlog();
			try
			{
				blog = BlogMLSerializer.Deserialize(new StringReader(XML));
			}
			catch (Exception ex)
			{
				throw new Exception("Couldn't load your BlogML file. Maybe it is wellformedness , Message:" + ex.ToString());
			}

			LoadBlogProperties(blog);

			Hashtable categoryHash = this.ReadCategories(blog);

			LoadPosts(blog, categoryHash);
			UpdateStats(blog);

			return string.Format("Post:{0} , Comment:{1} , TrackBack:{2} , Attachment:{3}",
				_BlogPostCount, _BlogCommentCount, _BlogTrackBackCount, _BlogAttachmentCount);
		}
		#endregion

		#region Private methods
		private void Clear()
		{
			ClearCategories();
			ClearPosts();
		}

		private void ClearCategories()
		{
			List<PostCategory> Categories = PostCategories.GetCategories(this._Blog.SectionID, true);

			foreach (PostCategory Category in Categories)
			{
				Category.IsEnabled = true;
				PostCategories.DeleteCategory(Category.CategoryID, Category.SectionID);
			}
		}

		private void ClearPosts()
		{
			ThreadSet blogThreads = new ThreadSet();
			BlogThreadQuery query = new BlogThreadQuery();
			query.SectionID = this._Blog.SectionID;
			query.PageSize = this._Blog.PostCount + this._Blog.ArticleCount;
			query.BlogPostType = BlogPostType.Post | BlogPostType.Article;
			blogThreads = WeblogPosts.GetBlogThreads(query);

			foreach (Post Thread in blogThreads.Threads)
			{
				WeblogPost Post = new WeblogPost();
				Post = WeblogPosts.GetPost(Thread.PostID, true, Thread.IsApproved, false);

				ClearPostComments(Post);
				ClearPostTrackBacks(Post);

				WeblogPosts.Delete(Post, CSContext.Current.User.UserID);
			}
		}

		private void CheckAndGetProperty(Pair<string, string> ExtendedProperty)
		{
			if (ExtendedProperty.Key == BlogMLBlogExtendedProperties.CommentModeration)
				this._Blog.ModerationTypeDefault = GetCommentModerationType(ExtendedProperty.Value);
			else if (ExtendedProperty.Key == BlogMLBlogExtendedProperties.EnableSendingTrackbacks)
				this._Blog.EnableTrackbacksDefault = GetSendTrackbackType(ExtendedProperty.Value);
			Weblogs.Update(this._Blog);
		}

		private void ClearPostComments(WeblogPost Post)
		{
			ThreadSet blogThreads = new ThreadSet();
			BlogThreadQuery query = new BlogThreadQuery();
			query.SectionID = this._Blog.SectionID;
			query.PageSize = this._Blog.CommentCount;
			query.BlogPostType = BlogPostType.Post | BlogPostType.Article;
			query.ParentID = Post.PostID;
			blogThreads = WeblogPosts.GetBlogThreads(query);

			foreach (Post Thread in blogThreads.Threads)
			{
				WeblogPost Comment = WeblogPosts.GetPost(Thread.PostID, true, Thread.IsApproved, false);
				WeblogPosts.Delete(Comment, CSContext.Current.User.UserID);
			}
		}

		private void ClearPostTrackBacks(WeblogPost Post)
		{
			ThreadSet blogThreads = new ThreadSet();
			BlogThreadQuery query = new BlogThreadQuery();
			query.SectionID = this._Blog.SectionID;
			query.PageSize = this._Blog.TrackbackCount;
			query.BlogPostType = BlogPostType.Trackback;
			query.ParentID = Post.PostID;
			blogThreads = WeblogPosts.GetBlogThreads(query);

			foreach (Post Thread in blogThreads.Threads)
			{
				WeblogPost TrackBack = WeblogPosts.GetPost(Thread.PostID, true, Thread.IsApproved, false);
				WeblogPosts.Delete(TrackBack, CSContext.Current.User.UserID);
			}
		}

		private CommentModerationType GetCommentModerationType(string strModerationType)
		{
			CommentModerationTypes ModerationType = (CommentModerationTypes)Enum.Parse(typeof(CommentModerationTypes), strModerationType, true);

			switch (ModerationType)
			{
				case CommentModerationTypes.Disabled:
					return CommentModerationType.Ignore;
				case CommentModerationTypes.Enabled:
					return CommentModerationType.All;
				case CommentModerationTypes.Anonymous:
					return CommentModerationType.Anonymous;
			}
			return CommentModerationType.None;
		}

		private bool GetSendTrackbackType(string strSendTrackbackType)
		{
			SendTrackbackTypes SendTrackbackType = (SendTrackbackTypes)Enum.Parse(typeof(SendTrackbackTypes), strSendTrackbackType, true);

			switch (SendTrackbackType)
			{
				case SendTrackbackTypes.Yes:
					return true;
				case SendTrackbackTypes.No:
					return false;
			}
			return true; ;
		}

		private void LoadAttachments(Post NewPost, BlogMLPost Post)
		{
			if (Post.Attachments.Count > 0)
			{
				CommonDataProvider provider = CommonDataProvider.Instance();

				PostAttachment attachment = new PostAttachment();
				if (Post.Attachments[0].Embedded)
				{
					// Local File , use CentralizedFile save file
					attachment.Length = (long)Post.Attachments[0].Data.Length;
					attachment.FileName = Post.Attachments[0].Url;
					attachment.IsRemote = false;
					attachment.ContentType = Post.Attachments[0].MimeType;
					attachment.SectionID = NewPost.SectionID;

					PostAttachments.Add(NewPost, attachment, new MemoryStream(Post.Attachments[0].Data));
				}
				else
				{
					// Remote File , save URL
					attachment.FileName = Post.Attachments[0].Url;
					attachment.IsRemote = true;
					attachment.ContentType = Post.Attachments[0].MimeType;
					attachment.SectionID = NewPost.SectionID;
				}
				provider.AddPostAttachment(NewPost, attachment);
				_BlogAttachmentCount++;
			}
		}

		private void LoadBlogProperties(BlogMLBlog Blog)
		{
			BlogMLBlog.ExtendedPropertiesCollection propertiess = Blog.ExtendedProperties;

			foreach (Pair<string, string> ExtendedProperty in propertiess)
			{
				CheckAndGetProperty(ExtendedProperty);
			}
		}

		private void LoadComments(Post NewPost, BlogMLPost Post)
		{
			if (Post.Comments.Count > 0)
			{
				foreach (BlogMLComment Comment in Post.Comments)
				{
					WeblogPost post = new WeblogPost();

					post.Body = Comment.Content.UncodedText;
					post.BlogPostType = BlogPostType.Comment;
					post.Subject = Comment.Title;
					post.ThreadDate = Comment.DateCreated;
					post.UserTime = Comment.DateCreated;
					post.PostDate = Comment.DateCreated;
					post.UserTime = Comment.DateCreated;
					post.SectionID = this._Blog.SectionID;
					post.SetExtendedAttribute("EverPublished", true.ToString());
					post.SetExtendedAttribute("TitleUrl", Comment.UserUrl);
					post.SetExtendedAttribute("SubmittedUserName", Comment.UserName);
					post.IsApproved = Comment.Approved;
					post.ParentID = NewPost.PostID;
					post.ThreadID = NewPost.ThreadID;
					post.PostLevel = 2;
					post.PostType = PostContentType.HTML;

					User user = new User();
					user.UserID = 2101;
					user.Username = "Anonymous";

					WeblogPosts.Add(post, user);

					// Update blog counter
					_BlogCommentCount++;
				}
			}
		}

		private void LoadPosts(BlogMLBlog Blog, Hashtable CategoryHash)
		{
			string str = null;
			int num = 0;
			DateTime time = new DateTime();
			string str2 = null;
			string str3 = null;

			foreach (BlogMLPost post2 in Blog.Posts)
			{
				WeblogPost newPost = new WeblogPost();
				newPost.Body = post2.Content.UncodedText;
				if (post2.PostType == BlogPostTypes.Normal)
					newPost.BlogPostType = BlogPostType.Post;
				else if (post2.PostType == BlogPostTypes.Article)
					newPost.BlogPostType = BlogPostType.Article;
				newPost.Subject = post2.Title;
				newPost.ThreadDate = post2.DateCreated;
				newPost.UserTime = post2.DateCreated;
				newPost.PostDate = post2.DateCreated;
				newPost.UserTime = post2.DateCreated;
				newPost.SectionID = this._Blog.SectionID;
				newPost.Views = (int)post2.Views;
				//newPost.Name = post2.PostName
				newPost.SetExtendedAttribute("EverPublished", post2.Approved.ToString());
				newPost.IndexInThread = 1; // true
				newPost.IsApproved = true;
				newPost.IsAggregated = true;
				newPost.IsLocked = false;
				newPost.IsAggregated = true;
				newPost.IsCommunityAggregated = true;
				newPost.PostLevel = 1;
				newPost.PostType = PostContentType.HTML;

				if (post2.Categories.Count > 0)
				{
					string[] strArray = new string[post2.Categories.Count + 1];
					int num3 = post2.Categories.Count - 1;
					for (int i = 0; i <= num3; i++)
					{
						strArray[i] = CategoryHash[post2.Categories[i].Ref].ToString();
					}
					newPost.Categories = strArray;
				}

				WeblogPosts.Add(newPost);

				// Temporary saving stats
				str = newPost.Username;
				num = newPost.AuthorID;
				time = newPost.PostDate;
				str2 = newPost.Name;
				str3 = newPost.Subject;

				LoadComments(newPost, post2);
				LoadTrackBacks(newPost, post2);
				LoadAttachments(newPost, post2);
				_BlogPostCount++;
			}

			// Update blog stats
			this._Blog.MostRecentPostAuthor = str;
			this._Blog.MostRecentPostAuthorID = num;
			this._Blog.MostRecentPostDate = time;
			this._Blog.MostRecentPostName = str2;
			this._Blog.MostRecentPostSubject = str3;
		}

		private void LoadTrackBacks(Post NewPost, BlogMLPost Post)
		{
			if (Post.Trackbacks.Count > 0)
			{
				foreach (BlogMLTrackback TrackBack in Post.Trackbacks)
				{
					WeblogPost post = new WeblogPost();
					post.Body = TrackBack.Title;
					post.BlogPostType = BlogPostType.Trackback;
					post.Subject = TrackBack.Title;
					post.ThreadDate = TrackBack.DateCreated;
					post.UserTime = TrackBack.DateCreated;
					post.PostDate = TrackBack.DateCreated;
					post.UserTime = TrackBack.DateCreated;
					post.SectionID = this._Blog.SectionID;
					post.SetExtendedAttribute("EverPublished", true.ToString());
					post.SetExtendedAttribute("TitleUrl", TrackBack.Url);
					post.SetExtendedAttribute("trackbackName", "TrackBack");
					post.IsApproved = TrackBack.Approved;
					post.ParentID = NewPost.PostID;
					post.ThreadID = NewPost.ThreadID;
					post.PostLevel = 2;
					post.PostType = PostContentType.Text;

					User user = new User();
					user.UserID = 2101;
					user.Username = "Anonymous";
					WeblogPosts.Add(post, user);

					// Update blog counter
					_BlogTrackBackCount++;
				}
			}
		}

		private Hashtable ReadCategories(BlogMLBlog Blog)
		{
			Hashtable hashtable = new Hashtable();
			foreach (BlogMLCategory category in Blog.Categories)
			{
				PostCategory category2 = new PostCategory();
				category2.DateCreated = category.DateCreated;
				category2.Description = category.Description;
				category2.IsEnabled = category.Approved;
				category2.Name = category.Title;
				category2.ParentID = 0;
				category2.SectionID = this._Blog.SectionID;
				try
				{
					PostCategories.CreateCategory(category2);
				}
				catch (Exception ex)
				{
					throw new ArgumentException("Error when tried to add categories to database , Message:" + ex.ToString());
				}
				hashtable.Add(category.ID, category.Title);
			}
			return hashtable;
		}

		private void UpdateStats(BlogMLBlog Blog)
		{
			Weblog weblog = this._Blog;
			weblog.PostCount = (weblog.PostCount + Blog.Posts.Count);
			weblog = this._Blog;
			weblog.CommentCount = (weblog.CommentCount + this._BlogCommentCount);
			weblog = this._Blog;
			weblog.TrackbackCount = (weblog.TrackbackCount + this._BlogTrackBackCount);
		}
		#endregion
	}
}
