﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using BlogML;
using BlogML.Xml;
using CommunityServer;
using CommunityServer.Components;
using CommunityServer.Blogs;
using CommunityServer.Blogs.Components;

namespace CommunityServer.BlogML
{
	public class Writer : BlogMLWriterBase
	{
		#region Private Members
		// Fields
		private Weblog _Blog;
		private int _CommentPageSize;
		private int _PostPageSize;
		private int _TrackBackPageSize;
		#endregion

		#region Constructor
		// Methods
		public Writer(string ApplicationKey)
		{
			this._Blog = Weblogs.GetWeblog(ApplicationKey);
			this._PostPageSize = 10 + (2 * (this._Blog.PostCount + this._Blog.ArticleCount));
			this._CommentPageSize = 10 + (2 * this._Blog.CommentCount);
			this._TrackBackPageSize = 10 + (2 * this._Blog.TrackbackCount);
		}
		#endregion

		#region Main process
		protected override void InternalWriteBlog()
		{
			WriteBlogInfo();
			WriteCategories();
			WritePosts();
			WriteEndElement();
		}
		#endregion

		#region Private methods
		private CommentModerationTypes GetCommentModerationProperty(CommentModerationType ModerationType)
		{
			switch (ModerationType)
			{
				case CommentModerationType.None:
					return CommentModerationTypes.Disabled;
				case CommentModerationType.All:
					return CommentModerationTypes.Enabled;
				case CommentModerationType.Anonymous:
					return CommentModerationTypes.Anonymous;
			}
			return CommentModerationTypes.Disabled;
		}

		private string GetPostUrl(WeblogPost Post)
		{
			return Post.ViewPostURL;
		}

		private SendTrackbackTypes GetSendTrackbackProperty(bool Send)
		{
			switch (Send)
			{
				case false:
					return SendTrackbackTypes.No;
				case true:
					return SendTrackbackTypes.Yes;
			}
			return SendTrackbackTypes.Yes;
		}

		private void WriteBlogAuthors()
		{
			base.WriteStartAuthors();
			if (this._Blog.OwnerArray.Length <= 0)
				throw new Exception("Blog must have at least one owner!");

			foreach (string str in this._Blog.OwnerArray)
			{
				User user = Users.FindUserByUsername(str);
				base.WriteAuthor(user.UserID.ToString(), user.Username, user.Email, user.DateCreated, user.LastLogin, !user.IsBanned);
			}
			base.WriteEndElement();
		}

		private void WriteBlogExtendedProperties()
		{
			base.WriteStartExtendedProperties();
			base.WriteExtendedProperty(BlogMLBlogExtendedProperties.CommentModeration, this.GetCommentModerationProperty(this._Blog.ModerationTypeDefault).ToString());
			base.WriteExtendedProperty(BlogMLBlogExtendedProperties.EnableSendingTrackbacks, this.GetSendTrackbackProperty(this._Blog.EnableTrackbacksDefault).ToString());
			base.WriteEndElement();
		}

		private void WriteBlogInfo()
		{
			base.WriteStartBlog(this._Blog.Name, this._Blog.Description, this._Blog.Url);
			this.WriteBlogAuthors();
			this.WriteBlogExtendedProperties();
		}

		private void WriteCategories()
		{
			List<PostCategory> categories = PostCategories.GetCategories(this._Blog.SectionID, true);
			if (categories.Count != 0)
			{
				base.WriteStartCategories();
				foreach (PostCategory category in categories)
				{
					PostCategory category2 = category;
					base.WriteCategory(category2.CategoryID.ToString(), category2.Name, category2.DateCreated, category2.DateCreated, category2.IsEnabled, category2.Description, category2.ParentID.ToString());
					category2 = null;
				}
				base.WriteEndElement();
			}
		}

		private void WritePostAttachments(WeblogPost Post)
		{
			if (Post.HasAttachment)
			{
				base.WriteStartAttachments();
				if (Post.Attachment.IsRemote)
				{
                    // Remote , Save URL
					string str = "none";
					if (!string.IsNullOrEmpty(Post.Attachment.ContentType))
						str = Post.Attachment.ContentType;

					base.WriteAttachment(Post.Attachment.FileName, str, Post.Attachment.FileName);
				}
				else
				{
                    // Local , Save stream
					PostAttachment postAttachment;
					postAttachment = CommonDataProvider.Instance().GetPostAttachment(Post.PostID);
                    base.WriteAttachment(postAttachment.FileName, postAttachment.ContentType, postAttachment.OpenReadStream());
				}
				base.WriteEndElement();
			}
		}

		private void WritePostAuthors(WeblogPost Post)
		{
			base.WriteStartAuthors();
			base.WriteAuthorReference(Post.AuthorID.ToString());
			base.WriteEndElement();
		}

		private void WritePostCategories(WeblogPost Post)
		{
			if (Post.HasCategories)
			{
				string[] strArray = Post.Categories;
				base.WriteStartCategories();
				foreach (string str in strArray)
				{
					PostCategory category = PostCategories.GetCategory(str, this._Blog.SectionID);
					base.WriteCategoryReference(category.CategoryID.ToString());
				}
				base.WriteEndElement();
			}
		}

		private void WritePostComments(WeblogPost Post)
		{
			ThreadSet blogThreads = new ThreadSet();
			BlogThreadQuery query = new BlogThreadQuery();
			query.SectionID = this._Blog.SectionID;
			query.PageSize = this._CommentPageSize;
			query.BlogPostType = BlogPostType.Comment;
			query.ParentID = Post.PostID;
			blogThreads = WeblogPosts.GetBlogThreads(query, false);
			if (blogThreads.Threads.Count != 0)
			{
				base.WriteStartComments();
				foreach (Post post in blogThreads.Threads)
				{
					Post post2 = post;
					string extendedAttribute = post2.GetExtendedAttribute("SubmittedUserName");
					string str2 = post2.GetExtendedAttribute("TitleUrl");
					if (string.IsNullOrEmpty(extendedAttribute))
						extendedAttribute = post2.Username;

					if (string.IsNullOrEmpty(extendedAttribute))
						extendedAttribute = "anonymous";

					if (string.IsNullOrEmpty(str2))
						str2 = "http://";

					base.WriteComment(post2.PostID.ToString(), post2.Subject, post2.PostDate, post2.PostDate, post2.IsApproved, extendedAttribute, null, str2, post2.Body);
					post2 = null;
				}
				this.WriteEndElement();
			}
		}

		private void WritePosts()
		{
			ThreadSet blogThreads = new ThreadSet();
			BlogThreadQuery query = new BlogThreadQuery();
			query.SectionID = this._Blog.SectionID;
			query.PageSize = this._PostPageSize;
			query.BlogPostType = BlogPostType.Post | BlogPostType.Article;
			blogThreads = WeblogPosts.GetBlogThreads(query);
			if (blogThreads.Threads.Count != 0)
			{
				base.WriteStartPosts();
				foreach (Post post in blogThreads.Threads)
				{
					BlogPostTypes types = BlogPostTypes.Normal;
					WeblogPost post2 = WeblogPosts.GetPost(post.PostID, true, post.IsApproved, false);
					if (post2.BlogPostType == BlogPostType.Post)
						types = BlogPostTypes.Normal;
					else if (post2.BlogPostType == BlogPostType.Article)
						types = BlogPostTypes.Article;

					string postUrl = this.GetPostUrl(post2);
					if (!post.HasExcerpt)
						base.WriteStartPost(post2.PostID.ToString(), post2.Subject, post2.PostDate, post2.PostDate, post2.IsApproved, post2.Body, postUrl, (uint)post2.Views, types, post2.Name);
					else
						base.WriteStartPost(post2.PostID.ToString(), post2.Subject, post2.PostDate, post2.PostDate, post2.IsApproved, post2.Body, postUrl, (uint)post2.Views, post.Excerpt, types, post2.Name);

					this.WritePostCategories(post2);
					this.WritePostComments(post2);
					this.WritePostTrackBacks(post2);
					this.WritePostAttachments(post2);
					this.WritePostAuthors(post2);
					base.WriteEndElement();
				}
				base.WriteEndElement();
			}
		}

		private void WritePostTrackBacks(WeblogPost Post)
		{
			ThreadSet blogThreads = new ThreadSet();
			BlogThreadQuery query = new BlogThreadQuery();
			query.SectionID = this._Blog.SectionID;
			query.PageSize = this._TrackBackPageSize;
			query.BlogPostType = BlogPostType.Trackback;
			query.ParentID = Post.PostID;
			blogThreads = WeblogPosts.GetBlogThreads(query, false);
			if (blogThreads.Threads.Count != 0)
			{
				base.WriteStartTrackbacks();
				foreach (Post post in blogThreads.Threads)
				{
					Post post2 = post;
					string extendedAttribute = post2.GetExtendedAttribute("TitleUrl");
					if (string.IsNullOrEmpty(extendedAttribute))
						extendedAttribute = post2.GetExtendedAttribute("TitleUrl");

					if (string.IsNullOrEmpty(extendedAttribute))
						extendedAttribute = "http://";

					base.WriteTrackback(post2.PostID.ToString(), post2.Subject.ToString(), post2.PostDate, post2.PostDate, post2.IsApproved, extendedAttribute);
					post2 = null;
				}
				base.WriteEndElement();
			}
		}
		#endregion
	}
}
