// dasBlog/BlogML conversion helper classes
// copyright (c) 2006 Paul van Brenk
// All rights reserved.
//
// BlogML classes are provided by BlogML project (www.blogml.com)
// dasblog classes are provided by dasBlog project (www.dasblog.net)
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
// OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
// IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// -------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Xml.Serialization;

using newtelligence.DasBlog.Runtime;
using newtelligence.DasBlog.Web;

using BlogML.Xml;

namespace DasBlog.BlogML {


    public class DasBlogMLReader {

        /// <summary>
        /// Creates a new instance of the <see cref="DasBlogMLReader"/> class.
        /// </summary>
        /// <param name="blogDataService">Dataservice for writing the contents of the destination blog.</param>
        /// <param name="siteConfig">Configuration of the destination blog.</param>
        /// <param name="siteSecurityConfig">Security configuration of the destination blog.</param>
        /// <param name="destination">Desitination root where to write the new content.</param>
        /// <exception cref="ArgumentNullException">Thrown when <paramref name="blogDataService"/> is <see langword="null"/>, 
        /// when <paramref name="siteConfig"/> is <see langword="null"/> or when <paramref name="siteSecurityConfig"/> is <see langword="null"/>.
        public DasBlogMLReader(IBlogDataService blogDataService, SiteConfig siteConfig, SiteSecurityConfig siteSecurityConfig)
            : base() {

            // parameter validation
            if (blogDataService == null) {
                throw new ArgumentNullException("blogDataService");
            }

            if (siteConfig == null) {
                throw new ArgumentNullException("siteConfig");
            }

            if (siteSecurityConfig == null) {
                throw new ArgumentNullException("siteSecurityConfig");
            }

            // setup fields
            this.blogDataService = blogDataService;
            this.siteConfig = siteConfig;
            this.siteSecurityConfig = siteSecurityConfig;
        }

        /// <summary>
        /// Reads the xml containing the blogml content and writes new dasBlog content.
        /// </summary>
        /// <param name="xml"></param>
        public void Read(string xml) {

            // parameter validation
            if (string.IsNullOrEmpty(xml)) {
                throw new ArgumentNullException("xml");
            }

            Read(new StringReader(xml));
        }

        /// <summary>
        /// Reads the reader containing the blogml content and writes new dasBlog content.
        /// </summary>
        /// <param name="stream"></param>
        public void Read(TextReader reader) {

            // parameter validation                                                         
            if (reader == null) {
                throw new ArgumentNullException("reader");
            }

            this.Read(BlogMLSerializer.Deserialize(reader));
        }

        /// <summary>
        /// Reads the BlogMLBlog instance containing the blogml content and writes the new dasBlog content.
        /// </summary>
        /// <param name="xml"></param>
        public void Read(BlogMLBlog blog) {

            // parameter validation
            if (blog == null) {
                throw new ArgumentNullException("blog");
            }

            // initialize categories collection, for easy access later
            Dictionary<string, string> categories = new Dictionary<string, string>(blog.Categories.Count);
            if (blog.Categories != null && blog.Categories.Count > 0) {
                foreach (BlogMLCategory category in blog.Categories) {
                    // dasBlog doesn't support parent/child relations between 
                    // categories so we flatten them
                    categories.Add(category.ID, category.Title);
                }
            }

            // process the posts
            if (blog.Posts != null && blog.Posts.Count > 0) {
                foreach (BlogMLPost post in blog.Posts) {

                    Entry entry = new Entry();

                    // re-use the old id, if we can, so we don't break any urls
                    Guid id;
                    try {
                        id = new Guid(post.ID);
                    } catch {
                        // all kinds of exceptions are to be expected
                        // oops, no guid. Let's make a new one
                        id = Guid.NewGuid();
                    }

                    entry.EntryId = id.ToString();

                    // set the dates 
                    entry.CreatedUtc = post.DateCreated;
                    entry.ModifiedUtc = post.DateModified;

                    // set the contents
                    entry.Title = post.Title;
                    entry.Description = post.Excerpt.Text;
                    entry.Content = post.Content.Text;
                    entry.IsPublic = post.Approved;

                    // find the author, if it's not the blog owner
                    if (post.Authors != null && post.Authors.Count > 0) {
                        if (post.Authors[0].Ref == siteSecurityConfig.Users[0].Name) {
                            BlogMLAuthor author = blog.Authors.Find(delegate(BlogMLAuthor current) {
                                return current.ID == post.Authors[0].Ref;
                            });

                            // add author if we found one, else use the ref.
                            entry.Author = author != null ? author.Title : post.Authors[0].Ref;
                        } 
                    }

                    // add the categories
                    if (post.Categories != null && post.Categories.Count > 0) {
                        StringBuilder catList = new StringBuilder();
                        foreach (BlogMLCategoryReference catRef in post.Categories) {
                            if (catList.Length > 0) { catList.Append(';'); }
                            catList.Append(categories[catRef.Ref]);
                        }
                        entry.Categories = catList.ToString();
                    }

                    // save the entry
                    if (blogDataService.SaveEntry(entry, null) == EntrySaveState.Failed) {
                        // that didn't work-out as we expected
                        // TODO: log this                         
                        continue;
                    }

                    // now add the comments
                    AddComments(post.Comments, entry);
                    // add the trackbacks
                    AddTrackbacks(post.Trackbacks, entry);
                    // add the attachments
                    AddAttachments(post.Attachments, entry);
                }
            }

        }

        private void AddAttachments(BlogMLPost.AttachmentCollection attachmentCollection, Entry entry) {

            if (attachmentCollection != null && attachmentCollection.Count > 0) {
                foreach (BlogMLAttachment mlAttachment in attachmentCollection) {
                    // Since we store both images linked inline in the post
                    // and rssEnclosures as attachments.
                    // We're in trouble :-(

                    // TODO: handle the attachments
                }
            }
        }

        private void AddComments(BlogMLPost.CommentCollection comments, Entry entry) {

            if (comments != null && comments.Count > 0) {

                foreach (BlogMLComment mlComment in comments) {

                    Comment dasComment = new Comment();
                    // author info
                    dasComment.Author = mlComment.UserName;
                    dasComment.AuthorEmail = mlComment.UserEMail;
                    dasComment.AuthorHomepage = mlComment.UserUrl;
                    // content
                    dasComment.Content = mlComment.Content.Text;
                    // dates
                    dasComment.CreatedUtc = mlComment.DateCreated;
                    dasComment.ModifiedUtc = mlComment.DateModified;
                    // status of the comment
                    dasComment.IsPublic = mlComment.Approved;
                    
                    // set id
                    // re-use the old id, if we can, so we don't break any urls
                    Guid id;
                    try {
                        id = new Guid(mlComment.ID);
                    } catch {
                        // all kinds of exceptions are to be expected
                        // oops, no guid. Let's make a new one
                        id = Guid.NewGuid();
                    }
                    dasComment.EntryId = id.ToString();
                    // set id of the post this comment belongs to
                    dasComment.TargetEntryId = entry.EntryId;
                    dasComment.TargetTitle = entry.Title;
                     
                    // save the comment
                    blogDataService.AddComment(dasComment);
                }
            }

        }


        private void AddTrackbacks(BlogMLPost.TrackbackCollection trackbackCollection, Entry entry) {
            if (trackbackCollection != null && trackbackCollection.Count > 0) {

                foreach (BlogMLTrackback mlTrackback in trackbackCollection) {

                    Tracking dasTrackback = new Tracking();
                    // blogml only supports trackbacks
                    dasTrackback.TrackingType = TrackingType.Trackback;
                    // set target properties
                    dasTrackback.TargetTitle = entry.Title;
                    dasTrackback.TargetEntryId = entry.EntryId;
                    // set source properties
                    dasTrackback.PermaLink = mlTrackback.Url;
                    dasTrackback.Referer = mlTrackback.Title;
                 
                    // save the trackback
                    blogDataService.AddTracking(dasTrackback);
                }
            }
        }



        // fields
        private IBlogDataService blogDataService;
        private SiteConfig siteConfig;
        private SiteSecurityConfig siteSecurityConfig;
    }
}
