// dasBlog/BlogML conversion helper classes
// copyright (c) 2006 Paul van Brenk
// All rights reserved.
//
// BlogML classes are provided by BlogML project (www.blogml.com)
// dasblog classes are provided by dasBlog project (www.dasblog.net)
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
// OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER 
// IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// -------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.IO; 
using System.Text;
using System.Web;

using BlogML;

using newtelligence.DasBlog.Web;
using newtelligence.DasBlog.Runtime;
using newtelligence.DasBlog.Web.Core;

using Property = DasBlog.BlogML.ExtendedProperties;


namespace DasBlog.BlogML {

    /// <summary>
    /// Converts the content from a dasBlog installation to a blogML file.
    /// <remarks>
    /// <ul>
    /// <li>No binary content is embedded; all rss enclosures and images in the content of a post are 
    /// stored as external attachements (i.e. a link to an external file).</li>
    /// <li>Specific language information for a post is not retained, only the general language for the blog.</li>
    /// <li>Only trackbacks are stored; pingbacks and referrals are lost.</li>
    /// </ul>
    /// </remarks>
    /// </summary>
    public class DasBlogMLWriter : BlogMLWriterBase {

        
        /// <summary>
        /// Creates a new instance of the <see cref="DasBlogMLWriter"/> class.
        /// </summary>
        /// <param name="blogDataService">Dataservice for reading the contents of the source blog.</param>
        /// <param name="siteConfig">Configuration of the source blog.</param>
        /// <param name="siteSecurityConfig">Security configuration of the source blog.</param>
        /// <exception cref="ArgumentNullException">Thrown when <paramref name="blogDataService"/> is <see langword="null"/>, 
        /// when <paramref name="siteConfig"/> is <see langword="null"/>, or when <paramref name="siteSecurityConfig"/> is <see langword="null"/> .</exception>
        public DasBlogMLWriter(IBlogDataService blogDataService, SiteConfig siteConfig, SiteSecurityConfig siteSecurityConfig)
            : base() {

            // parameter validation
            if (blogDataService == null) {
                throw new ArgumentNullException("blogDataService");
            }

            if (siteConfig == null) {
                throw new ArgumentNullException("siteConfig");
            }

            if (siteSecurityConfig == null) {
                throw new ArgumentNullException("siteSecurityConfig");
            }

            // setup fields
            this.blogDataService = blogDataService;
            this.siteConfig = siteConfig;
            this.siteSecurityConfig = siteSecurityConfig;
        }

        /// <summary>
        /// Actual writing happens here.
        /// </summary>
        protected override void InternalWriteBlog() {

            // make sure we have a writer
            if (Writer == null) {
                throw new InvalidOperationException("XmlWriter is null.");
            }

            // datecreated = datetime.now since we don't store that.
            WriteStartBlog(siteConfig.Title, ContentTypes.Text, siteConfig.Subtitle, ContentTypes.Text, siteConfig.Root, DateTime.Now);


            WriteStartAuthors();

            // assume we have one author
            UserCollection authors = siteSecurityConfig.Users;
            if (authors != null) {
                foreach (User author in authors) {
                    WriteAuthor(HttpUtility.HtmlEncode( author.Name), author.DisplayName, author.EmailAddress, DateTime.MinValue, DateTime.MinValue, true);
                }
            }
            WriteEndElement(); // end authors

            WriteStartExtendedProperties();
            WriteExtendedProperty(Property.Language, siteConfig.RssLanguage);
            WriteEndElement(); // extended properties

            // if the role of the current prinicipal is not admin we export only the public entries
            CategoryCacheEntryCollection categories = blogDataService.GetCategories();

            WriteStartCategories();

            foreach (CategoryCacheEntry category in categories) {
                WriteCategory(HttpUtility.HtmlEncode( category.Name), category.DisplayName, ContentTypes.Text, DateTime.MinValue, DateTime.MinValue, true, null, null);
            }

            WriteEndElement(); // end categories

            WriteStartPosts();

            // get all entries
            EntryCollection entries = blogDataService.GetEntries(null, null, Int32.MaxValue, Int32.MaxValue);

            foreach (Entry post in entries) {
                WritePost(post);
            }
            WriteEndElement(); // end posts

            // End Blog Element
            WriteEndElement();
            // clean up
            this.Writer.Flush();
        }

        // writes a post,including comments, trackbacks, attachements.
        private void WritePost(Entry post) {

            // write the post
            WriteStartPost(
                post.EntryId,
                post.Title,
                post.CreatedUtc,
                post.ModifiedUtc,
                post.IsPublic,
                post.Content,
                Utils.GetPermaLinkUrl(siteConfig, post),
                (uint)0, /* we don't support view count */
                post.Description,
                BlogPostTypes.Normal,
             "" /* we don't support post name */
                );

            // add categories
            WriteStartCategories();
            if (post.Categories != null && post.Categories.Length > 0) {
                foreach (string s in post.GetSplitCategories()) {
                    WriteCategoryReference(HttpUtility.HtmlEncode(s));
                }
            }

            WriteEndElement();// end categories

            // Add the comments
            WriteStartComments();

            // get all comments 
            CommentCollection comments = blogDataService.GetCommentsFor(post.EntryId, true);
            if (comments != null) {
                foreach (Comment comment in comments) {
                    WriteComment(comment.EntryId, null, comment.CreatedUtc, comment.ModifiedUtc, comment.IsPublic, comment.Author, comment.AuthorEmail, comment.AuthorHomepage, comment.Content);
                }
            }

            WriteEndElement(); // end comments

            // add the trackbacks
            WriteStartTrackbacks();

            TrackingCollection trackbacks = blogDataService.GetTrackingsFor(post.EntryId);
            if (trackbacks != null) {
                foreach (Tracking trackback in trackbacks) {
                    // we only store tracknacks here
                    if (trackback.TrackingType == TrackingType.Trackback) {
                        // create our own guid, since dasBlog doesn't add id's for trackbacks
                        // datetime.minvalue since dasBlog doesn't store the date/time of the trackback
                        WriteTrackback(Guid.NewGuid().ToString(), trackback.RefererTitle, DateTime.MinValue, DateTime.MinValue, true, trackback.PermaLink);
                    }
                }
            }

            WriteEndElement(); // end trackbacks

            // add the attachements
            WriteStartAttachments();

            // for now we don't embed the attachments
            AttachmentCollection attachments = post.Attachments;
            if (attachments != null) {
                foreach (Attachment attachment in attachments) {

                    string fullUrl = Utils.GetEnclosureLinkUrl(siteConfig, post.EntryId, attachment);
                    WriteAttachment(fullUrl, attachment.Type, fullUrl);
                }
            }

            // get the images from the post content
            // not embeded 
            string[] imageUrls = SgmlUtil.GetAttributeValues(post.Content, "img", "src");
            if (imageUrls != null && imageUrls.Length > 0) {

                foreach (string imageUrl in imageUrls) {
                    // only local urls
                    if (SgmlUtil.IsRootUrlOf(siteConfig.Root, imageUrl)) {
                        WriteAttachment(imageUrl, GetMimeType(imageUrl), imageUrl);
                    }
                }
            }

            WriteEndElement(); // end attachements

            // add the authors, eiter the post author or when not specified the blog owner
            WriteStartAuthors();
            if (post.Author != null && post.Author.Length != 0) {
                WriteAuthorReference(HttpUtility.HtmlEncode(post.Author));
            } else {
                WriteBlogOwnerReference();
            }
            WriteEndElement(); // end authors

            WriteEndElement(); // end post
        }

        // returns the mimetype for a given url
        // currently only supports a limited subset of images
        private static string GetMimeType(string url) {

            if (String.IsNullOrEmpty(url)) {
                return "";
            }
            string extension = Path.GetExtension(url);

            switch (extension) {
                case ".png":
                    return "image/png";
                case ".gif":
                    return "image/gif";
                case ".jpg":
                case ".jpeg":
                    return "image/jpg";
                default:
                    return "application/octet-stream";
            }
        }


        /// <summary>
        /// Writes an author reference for the owner of the blog.
        /// </summary>
        private void WriteBlogOwnerReference() {
            WriteAuthorReference(HttpUtility.HtmlEncode( this.siteSecurityConfig.Users[0].Name));
        }

        // fields

        private SiteConfig siteConfig;
        private SiteSecurityConfig siteSecurityConfig;
        private IBlogDataService blogDataService;
    }
}
