using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using BlogML;
using LiveSpace.BlogML.MetaWeblogApi;

namespace LiveSpace.BlogML
{
    public class LiveSpaceBlogMLWriter : BlogMLWriterBase
    {
        private string _liveSpaceName;
        private string _secretWord;

        public const int MaximumPostsSupportedByApi = 20; // limitation of the MSN Spaces MetaWeblog API
        private int _postCount;

        public LiveSpaceBlogMLWriter(string liveSpaceName, string secretWord)
            : base()
        {
            if (String.IsNullOrEmpty(liveSpaceName)) throw new ArgumentNullException("liveSpaceName");
            if (String.IsNullOrEmpty(secretWord)) throw new ArgumentNullException("secretWord");
            _liveSpaceName = liveSpaceName;
            _secretWord = secretWord;
            _postCount = MaximumPostsSupportedByApi;
        }

        /// <summary>
        /// Specified the number of posts to attempt to retrieve.
        /// </summary>
        /// <remarks>
        /// If the number exceeds MaximumPostsSupportedByApi, a much slower
        /// method is used for posts beyond that limit. It is recommended
        /// that you do not specify a number larger than the total number
        /// of posts on the blog.
        /// </remarks>
        public int PostCount
        {
            get
            {
                return _postCount;
            }
            set
            {
                if (value < 1) throw new ArgumentOutOfRangeException("PostCount");
                _postCount = value;
            }
        }

        protected override void InternalWriteBlog()
        {
            MsnSpacesMetaWeblog mw = new MsnSpacesMetaWeblog();
            string appKey = this.GetType().FullName;

            UserBlog[] blogs = mw.getUsersBlogs(appKey, _liveSpaceName, _secretWord);
            if (blogs.Length == 0) throw new InvalidOperationException("Cannot retrieve blog name.");

            WriteStartBlog(blogs[0].blogName, string.Empty, blogs[0].url);

            // author
            WriteStartAuthors();
            UserInfo user = mw.getUserInfo(appKey, _liveSpaceName, _secretWord);
            string userName = user.firstname + ' ' + user.lastname;
            WriteAuthor(user.userid, userName, user.email, DateTime.MinValue, DateTime.MinValue, true);
            WriteEndElement();

            // categories
            WriteStartCategories();
            Category[] categories = mw.getCategories(blogs[0].blogid, _liveSpaceName, _secretWord);
            foreach (Category c in categories)
            {
                WriteCategory(HttpUtility.HtmlEncode(c.title), c.title, DateTime.MinValue, DateTime.MinValue, true, c.description, null);
            }
            WriteEndElement();

            // posts
            WriteStartPosts();
            int numberOfPosts = Math.Min(PostCount, MaximumPostsSupportedByApi);
            Post[] posts = mw.getRecentPosts(blogs[0].blogid, _liveSpaceName, _secretWord, numberOfPosts);
            foreach (Post p in posts)
            {
                WritePost(user, p);
            }
            if (posts.Length > 0 && posts.Length < PostCount)
            {
                GetMorePosts(mw, user, posts);
            }
            WriteEndElement(); // end posts
                
            WriteEndElement(); // end blog

            this.Writer.Flush();
        }

        private void GetMorePosts(MsnSpacesMetaWeblog mw, UserInfo user, Post[] posts)
        {
            int count = posts.Length;
            Post lastPost = posts[posts.Length - 1];
            string postid = GuessPreviousPostId(lastPost.postid);
            while (postid != null && count < PostCount)
            {
                try
                {
                    Post p = mw.getPost(postid, _liveSpaceName, _secretWord);
                    WritePost(user, p);
                    count++;
                }
                catch (CookComputing.XmlRpc.XmlRpcFaultException ex)
                {
                    switch (ex.FaultCode)
                    {
                        case 3001: // Access Denied.
                        case 40003: // Directory or File Does Not Exist.
                        case 50605: // Item does not match expected type.
                            break;
                        default:
                            throw;
                    }
                }
                postid = GuessPreviousPostId(postid);
            }
        }

        private string GuessPreviousPostId(string postId)
        {
            if (postId == null) throw new ArgumentNullException("postId");
            const char sep = '!';
            if (postId.IndexOf(sep) < 0) return null;
            string[] parts = postId.Split(sep);
            if (parts.Length != 2) return null;
            int tail = 0;
            if (!int.TryParse(parts[1], out tail)) return null;
            tail--;
            if (tail < 0) return null;
            parts[1] = tail.ToString();
            return string.Join(sep.ToString(), parts);
        }

        private void WritePost(UserInfo user, Post p)
        {
            WriteStartPost(p.postid, p.title, p.dateCreated, p.dateCreated, p.publish, p.description, null);
            WriteStartCategories();
            foreach (string categoryName in p.categories)
            {
                WriteCategoryReference(HttpUtility.HtmlEncode(categoryName));
            }
            WriteEndElement(); // end categories
            WriteStartAuthors();
            WriteAuthorReference(user.userid);
            WriteEndElement(); // end authors
            WriteEndElement(); // end post
        }
    }
}
