using System;
using System.Collections;
using System.IO;
using System.Xml;
using MarkItUp.SingleUserBlog.Services;
using MarkItUp.SingleUserBlog.UI;
using MarkItUp.SingleUserBlog.Common;
using BlogML;
using BlogML.Xml;

namespace MarkItUp.SingleUserBlog.UI {
    public sealed class SUBBlogMLWriter : BlogMLWriterBase {

       protected override void InternalWriteBlog() {

            this.WriteStartBlog(
                BlogConfigurationSettings.BlogTitle,
                BlogConfigurationSettings.BlogSubTitle,
                Globals.UrlStartPath,
                DateTime.Now
                );
            this.WriteAuthor(Globals.UserLoginName, Globals.UserLoginName, Globals.AdministratorEmail, DateTime.MinValue, DateTime.MinValue, true);
            
            // adding categories
            this.WriteStartCategories();
            foreach (Category category in DataLayer.Instance.ListCategories()) {
                this.WriteCategory(
                    category.Id,
                    category.DisplayName,
                    category.DateCreated,
                    category.DateModified,
                    category.IsApproved,
                    category.Description,
                    category.ParentId);
            }
            this.WriteEndElement(); // categories


            // adding posts
            this.WriteStartPosts();

            PostCollection posts = DataLayer.Instance.ListPostsByAll();
            posts.Sort(new PostAddedDateComparer(true));

            foreach (MarkItUp.SingleUserBlog.Services.Post post in posts) {
                this.WriteStartPost(
                    post.Id,
                    post.DisplayName,
                    post.DateCreated,
                    post.DateModified,
                    post.IsApproved,
                    post.Description,
                    UrlFormatter.FormatUri(post),
                    (uint)0, /* Post Count not used by SUB */
                    BlogPostTypes.Normal,
                    string.Empty /* Post Name not used by SUB */
                    );

                // adding categories
                CategoryCollection categories = DataLayer.Instance.ListCategoriesByPost(post.Id);
                if (categories.Count > 0) {
                    this.WriteStartCategories();
                    foreach (Category category in categories)
                        this.WriteCategoryReference(category.Id);
                    this.WriteEndElement();
                }

                // adding comments
                FeedbackItemCollection feedbacks = DataLayer.Instance.ListFeedbackByPost(post.Id);
                if (feedbacks.Count > 0) {
                    this.WriteStartComments();
                    foreach (FeedbackItem feedback in feedbacks) {
                        string title = feedback.DisplayName;
                        if (title == null || title.Length == 0)
                            title = "No title entered";



                        this.WriteComment(
                            feedback.Id,
                            title,
                            feedback.DateCreated,
                            feedback.DateModified,
                            feedback.IsApproved,
                            feedback.Username,
                            null,
                            feedback.UserUrl,
                            feedback.Description
                            );
                    }
                    this.WriteEndElement();
                }


                // get all image sources and adding them as attachment
                // non embedded
                string[] imageUrls = SgmlUtil.GetAttributeValues(post.Description, "img", "src");
                if (imageUrls.Length > 0) {
                    this.WriteStartAttachments();

                    for (int i = 0; i < imageUrls.Length; i++) {
                        string imageUrl = imageUrls[i].ToLower();

                        if (SgmlUtil.IsRootUrlOf(Globals.UrlStartPath, imageUrl))
                            try {
                                string fullUrl = MakeFullUrl(imageUrl);
                                this.WriteAttachment(imageUrl, GetMimeType(fullUrl), fullUrl);
                            } catch (Exception ex) {
                                Console.WriteLine(ex.Message);
                            }
                    }

                    this.WriteEndElement();
                }


                this.WriteEndElement(); // post
            }
            this.WriteEndElement(); // posts

            this.WriteEndElement(); // blog
        }



        private string GetMimeType(string fullUrl) {

            string extension = Path.GetExtension(fullUrl);
            if (string.IsNullOrEmpty(extension)) {
                return "";
            }

            extension = extension.TrimStart(new char[] { '.' });

            string retVal = "none";

            switch (extension.ToLower()) {
                case "png":
                    retVal = "png";
                    break;
                case "jpg":
                case "jpeg":
                    retVal = "jpg";
                    break;
                case "bmp":
                    retVal = "bmp";
                    break;
                default:
                    retVal = "none";
                    break;
            }

            return retVal;
        }


        string MakeFullUrl(string imageUrl) {

            if (imageUrl.StartsWith(Globals.UrlStartPath)) {
                // already fully qualified
                return imageUrl;
            }

            // baseUrl will be "" or VirtualDirName/
            string baseUrl = Globals.UrlBasePage.TrimStart(new char[] { '/' }).ToLower();
            imageUrl = imageUrl.TrimStart(new char[] { '/' }).ToLower();


            // the imageUrl could be root relative ie: /Images/Foo.jpg or, /VirtualDirName/Images.Foo.jpg
            if (imageUrl.StartsWith(baseUrl)) {
                imageUrl = imageUrl.Substring(baseUrl.Length);
                //else if (!imageUrl.StartsWith("/") && !baseUrl.EndsWith("/")) {
                //    baseUrl = baseUrl + "/";
                //}

                imageUrl = Globals.UrlStartPath + imageUrl;
            }
            return imageUrl;
        }
    }
}
